'	Settings class
'	--	
'	Read and write settings to an XML .config file. Does not use 
'	ConfigurationSettings.AppSettings since it's not supported on 
'	.NET Compact Framework. 
'
'	Uses same schema as app.config file. Example:
'
'		<configuration>
'			<appSettings>
'				<app key="Name" value="Live Oak" />
'				<app key="LogEvents" value="True" />
'			</appSettings>
'		</configuration>	
'
'	Default settings file name is the same as app.config, 
'	appends .config to the end of the assembly name. Example:
'
'		<appname.exe>.config

Imports System.Xml
Imports System.IO


' Read and write .config settings.
Public Class Settings
	' internal fields
	Private _list As New Hashtable
	Private _filePath As String = ""
	Private _autoWrite As Boolean = True
    Private _defaultValues(,) As String

    ' Specifies if the settings file is updated whenever a value 
    ' is set. If false, you need to call Write to update the 
    ' underlying settings file.
    Public Property AutoWrite() As Boolean
        Get
            Return _autoWrite
        End Get

        Set(ByVal Value As Boolean)
            _autoWrite = Value
        End Set
    End Property

    ' Full path to settings file.
    Public Property FilePath() As String
        Get
            Return _filePath
        End Get

        Set(ByVal Value As String)
            _filePath = Value
        End Set
    End Property

    ' Default constructor. 
    Public Sub New()
        ' get full path to file
        _filePath = GetFilePath()

        ' populate list with settings from file
        Read()
    End Sub


    ' Constructor. Pass in an array of default values.
    ' Sample usage that passes in default values:
    '
    '		string[,] values = 
    '		{
    '			{"Name", "Live Oak"},
    '			{"LogEvents", "True"}
    '		}
    '
    '		Settings settings = new Settings(values);
    Public Sub New(ByVal defaultValues(,) As String)
        ' store default values, use later when populate list
        _defaultValues = defaultValues

        ' get full path to file
        _filePath = GetFilePath()

        ' populate list with settings from file
        Read()
    End Sub

    ' Set setting value. Update	underlying file	if AutoUpdate is true.
    Public Sub SetValue(ByVal key As String, ByVal value As Object)
        ' update internal list
        _list(key) = value

        ' update settings file
        If _autoWrite Then
            Write()
        End If
    End Sub

    ' Return specified settings	as string.
    Public Function GetString(ByVal key As String) As String
        Dim result As Object = _list(key)
        If result Is Nothing Then
            Return ""
        Else
            Return result.ToString()
        End If
    End Function

    ' Return specified settings as integer.
    Public Function GetInt(ByVal key As String) As Integer
        Dim result As String = GetString(key)
        If result = "" Then
            Return 0
        Else
            Return Convert.ToInt32(result)
        End If
    End Function

    ' Return specified settings as boolean.
    Public Function GetBool(ByVal key As String) As Boolean
        Dim result As String = GetString(key)
        If result = "" Then
            Return False
        Else
            Return Convert.ToBoolean(result)
        End If
    End Function

    ' Read settings file.
    Public Sub Read()
        ' first remove all items from list
        _list.Clear()

        ' next, populate list with default values
        Dim i As Integer
        For i = 0 To (_defaultValues.GetLength(0)) - 1
            _list(_defaultValues(i, 0)) = _defaultValues(i, 1)
        Next i

        ' last, populate list with items from file
        ' open settings file
        If File.Exists(_filePath) Then
            Dim reader As New XmlTextReader(_filePath)

            ' go through file and read the xml file and 
            ' populate internal list with 'add' elements
            While reader.Read()
                If reader.NodeType = XmlNodeType.Element And reader.Name = "app" Then
                    _list(reader.GetAttribute("key")) = reader.GetAttribute("value")
                End If
            End While

            reader.Close()
        End If
    End Sub


    ' Write settings to the .config file.
    Public Sub Write()
        ' header elements
        Dim writer As StreamWriter = File.CreateText(_filePath)
        writer.WriteLine("<configuration>")
        writer.WriteLine(ControlChars.Tab + "<appSettings>")

        ' go through internal list and create 'add' element for each item
        Dim enumerator As IDictionaryEnumerator = _list.GetEnumerator()
        While enumerator.MoveNext()
            writer.WriteLine(ControlChars.Tab + ControlChars.Tab + "<app key=""{0}"" value=""{1}"" />", enumerator.Key.ToString(), enumerator.Value)
        End While

        ' footer elements
        writer.WriteLine(ControlChars.Tab + "</appSettings>")
        writer.WriteLine("</configuration>")

        writer.Close()
    End Sub

    ' Overwrite settings with default values.
    Public Sub RestoreDefaults()
        Try
            ' easiest way is to delete the file and
            ' repopulate internal list with defaults
            File.Delete(FilePath)
            Read()
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try
    End Sub

    ' Return full path to settings file. Appends .config to the assembly name.
    Private Function GetFilePath() As String
        Return System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase + ".config"
    End Function

End Class
